*******************************
*Multilevel Workshop II Examples
*June 27/28, 2013
*StatLab@UVa Library
*Michele Claibourn
*******************************

log using "J:\public_html\Multilevel2Workshop\Multilevel2logfile.log", replace
***The Linear Mixed-Effects Model
use "C:\Users\mpc8t\Downloads\schools.dta", clear
gen csesbycath=cses*catholic
gen csesbymean=cses*meanses
xtmixed mathach cses catholic meanses csesbycath csesbymean || school: cses, variance covar(un)
est store lme1


***Exploratory Data Analysis and Model Building
reg mathach cses female minority
**Caterpillar plot
sort school
egen groupsize = count(id), by(school)
by school : gen first=_n==1
statsby intercept=_b[_cons] sesslope=_b[cses] femslope=_b[female] minslope=_b[minority] interceptse=_se[_cons] sesslopese=_se[cses] femslopese=_se[female] minslopese=_se[minority],  /// 
	by(school) saving(ols, replace): regress mathach cses female minority
sort school
merge school using ols
drop _merge
sort intercept
gen int_rank = sum(first)
serrbar intercept interceptse int_rank if first==1, scale(1.96) yline(12.76099)
sort sesslope
gen sesslope_rank = sum(first)
serrbar sesslope sesslopese sesslope_rank if first==1, scale(1.96) yline(2.191087)
sort femslope
gen  femslope_rank = sum(first)
serrbar femslope femslopese femslope_rank if first==1, scale(1.96) yline(2.191087)
sort minslope
gen minslope_rank = sum(first)
serrbar minslope minslopese minslope_rank if first==1, scale(1.96) yline(2.191087)

**Scatterplots
*By groups
graph twoway lfit mathach cses || lowess mathach cses || scatter mathach cses || if school<2500, by(school)
graph twoway lfit mathach minority || lowess mathach minority || scatter mathach minority, jitter(5) || if school<2500, by(school)

*By categorical groupl-level covariate
graph twoway scatter mathach cses, mcolor(gs10) msymbol(oh) || lfit mathach cses || lowess mathach cses || , by(catholic)
graph twoway scatter mathach female, jitter(5) mcolor(gs10) msymbol(oh) || lfit mathach female || lowess mathach female ||, by(catholic)

*By continuous group-level covariate
*Categorize the continuous variable meanses
egen meanses4=cut(meanses), group(4)
tab meanses4
graph twoway scatter mathach cses, mcolor(gs10) msymbol(oh) || lfit mathach cses || lowess mathach cses, by(meanses4) || , by(meanses4)

*Or sort the caterpillar plot by another variable
serrbar intercept interceptse meanses if first==1, scale(1.96) yline(12.76099)
serrbar sesslope sesslopese meanses if first==1, scale(1.96) yline(2.191087)
serrbar femslope femslopese meanses if first==1, scale(1.96) yline(2.191087)
serrbar minslope minslopese meanses if first==1, scale(1.96) yline(2.191087)

**Distributions
*Level-1 distribution (y, epsilon)
gen yhat=intercept + sesslope*cses + femslope*female + minslope*minority
gen olserror = mathach-yhat
hist olserror

*Random coefficients (beta)
hist intercept
kdensity sesslope
qnorm femslope
qnorm femslope if femslope~=0
kdensity minslope
kdensity minslope if minslope~=0

*kdens2 is a user-contributed package; to install type: ssc install kdens2
kdens2 intercept sesslope if first==1, levels(5)
kdens2 intercept femslope if first==1 & femslope~=0, levels(5)
kdens2 intercept minslope if first==1 & minslope~=0, levels(5)


***Estimation
**Full ML and REML 
xtmixed mathach cses catholic meanses csesbycath csesbymean || school: cses, variance covar(un)
xtmixed mathach cses catholic meanses csesbycath csesbymean || school: cses, variance covar(un) reml
est store rcoef
predict error1, residuals

**The BLUPs, as deviations from common CONDITIONAL parameter
predict b*, reffects
predict se*, reses
egen tag = tag(school)
serrbar b2 se2 school if tag==1, scale(1.96) yline(0)

sort b2
gen b2_rank = sum(tag)
serrbar b2 se2 b2_rank if tag==1, scale(1.96) yline(0)
graph save caterpillarintercept, replace
serrbar b1 se1 b2_rank if tag==1, scale(1.96) yline(0)
graph save caterpillarslope, replace

graph combine caterpillarintercept.gph caterpillarslope.gph, cols(1)

**Variation in parameters as a function of Z's and u's
scalar grandint = _b[_cons]
scalar grandslope = _b[cses]

gen schoolint = grandint + _b[catholic]*catholic + _b[meanses]*meanses + b2
gen schoolslope = grandslope + _b[csesbycath]*csesbycath + _b[csesbymean]*csesbymean + b1

sort schoolint
gen schoolint_rank = sum(tag)
serrbar schoolint se2 schoolint_rank if tag==1, scale(1.96) yline(`=grandint')
sort schoolslope
gen schoolslope_rank = sum(tag)
serrbar schoolslope se1 schoolslope_rank if tag==1, scale(1.96) yline(`=grandslope')


***Inference
**Fixed effects
*Are a set of level-2 predictors for a particular level-1 parameter jointly zero
test csesbycath csesbymean
*Is cses significant within Catholic schools?
test cses+csesbycath=0

**Variance components
*Is Random intercept significant?
xtmixed mathach cses catholic meanses csesbycath csesbymean || school: , variance covar(un) reml
est store rint

*Is covariance between random intercept and slope significant?
quietly xtmixed mathach cses catholic meanses csesbycath csesbymean || school: cses, variance covar(ind) reml
est store rcoefnocovar
lrtest rcoef rcoefnocovar

*Is random slope significant
lrtest rcoefnocovar rint
lrtest rcoef rint

**Variance parameters near zero
xtmixed mathach cses catholic female minority meanses csesbycath csesbymean || school: cses female minority, variance covar(un) reml
xtmixed mathach cses catholic female minority meanses csesbycath csesbymean || school: cses female minority, variance covar(un) 
xtmixed mathach cses catholic female minority meanses csesbycath csesbymean || school: female minority, variance covar(un) reml


**Model Comparison, Assessment
*Error assumptions
kdensity error1
qnorm b1
kdensity b2
kdens2 b2 b1 if tag==1, levels(5)

*Shrinkage (toward pooled estimate)
quietly xtmixed mathach cses female minority || school: cses, covar(un)
predict u1 u0, reffects
generate schoolslope2 = _b[cses] + u1
generate schoolint2 = _b[_cons] + u0
twoway pcarrow sesslope intercept schoolslope2 schoolint2 if tag==1
twoway pcarrow sesslope intercept schoolslope2 schoolint2 if tag==1 & school<2500


***Generalizations
**Load Hospital-Doctor-Patient simulated data
use "C:\mpc\DataServices\Workshops\MultilevelWorkshops\hdp.dta", clear
summ
*Three-level model
xtmixed mobility age sex bmi  || hid: || did: 
xtmixed mobility age sex bmi experience i.school medicaid || hid: || did:

**Mixed-effects logit
xtmelogit remission age lengthofstay i.familyhx il6 i.cancerstage experience || did: 

log close
